@@ -1,24 +1,24 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2012 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
  *   Free Software Foundation; either version 2 of the License, or (at your
  *   option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *   See the GNU General Public License for more details.
- * 
+ *
  *   You should have received a copy of the GNU General Public License along
  *   with this program; if not, see <http://www.gnu.org/licenses>.
- * 
+ *
  *   Linking stunnel statically or dynamically with other modules is making
  *   a combined work based on stunnel. Thus, the terms and conditions of
  *   the GNU General Public License cover the whole combination.
- * 
+ *
  *   In addition, as a special exception, the copyright holder of stunnel
  *   gives you permission to combine stunnel with free software programs or
  *   libraries that are released under the GNU LGPL and with code included
@@ -26,7 +26,7 @@
  *   modified versions of such code, with unchanged license). You may copy
  *   and distribute such a system following the terms of the GNU GPL for
  *   stunnel and the licenses of the other code concerned.
- * 
+ *
  *   Note that people who make modified versions of stunnel are not obligated
  *   to grant this special exception for their modified versions; it is their
  *   choice whether to do so. The GNU General Public License gives permission
@@ -42,15 +42,15 @@
 
 #include <tcpd.h>
 
-static int check(char *, int);
+NOEXPORT int check(char *, int);
 
 int allow_severity=LOG_NOTICE, deny_severity=LOG_WARNING;
 
 #ifdef USE_PTHREAD
 #define SERVNAME_LEN 256
 
-static ssize_t read_fd(int, void *, size_t, int *);
-static ssize_t write_fd(int, void *, size_t, int);
+NOEXPORT ssize_t read_fd(int, void *, size_t, int *);
+NOEXPORT ssize_t write_fd(int, void *, size_t, int);
 
 int num_processes=0;
 static int *ipc_socket, *busy;
@@ -156,7 +156,7 @@ void libwrap_auth(CLI *c, char *accepted_address) {
         s_log(LOG_DEBUG, "Acquired libwrap process #%d", my_process);
         write_fd(ipc_socket[2*my_process], c->opt->servname,
             strlen(c->opt->servname)+1, c->local_rfd.fd);
-        read_blocking(c, ipc_socket[2*my_process],
+        s_read(c, ipc_socket[2*my_process],
             (u8 *)&result, sizeof result);
         s_log(LOG_DEBUG, "Releasing libwrap process #%d", my_process);
 
@@ -168,13 +168,11 @@ void libwrap_auth(CLI *c, char *accepted_address) {
         }
         busy[my_process]=0; /* mark the child process as free */
         --num_busy; /* the child process has been released */
-        if(num_busy==num_processes-1) { /* need to wake up a thread */
-            retval=pthread_cond_signal(&cond); /* signal waiting threads */
-            if(retval) {
-                errno=retval;
-                ioerror("pthread_cond_signal");
-                longjmp(c->err, 1);
-            }
+        retval=pthread_cond_signal(&cond); /* signal a waiting thread */
+        if(retval) {
+            errno=retval;
+            ioerror("pthread_cond_signal");
+            longjmp(c->err, 1);
         }
         retval=pthread_mutex_unlock(&mutex);
         if(retval) {
@@ -201,7 +199,7 @@ void libwrap_auth(CLI *c, char *accepted_address) {
         c->opt->servname, accepted_address);
 }
 
-static int check(char *name, int fd) {
+NOEXPORT int check(char *name, int fd) {
     struct request_info request;
 
     request_init(&request, RQ_DAEMON, name, RQ_FILE, fd, 0);
@@ -211,7 +209,7 @@ static int check(char *name, int fd) {
 
 #ifdef USE_PTHREAD
 
-static ssize_t read_fd(int fd, void *ptr, size_t nbytes, int *recvfd) {
+NOEXPORT ssize_t read_fd(int fd, void *ptr, size_t nbytes, int *recvfd) {
     struct msghdr msg;
     struct iovec iov[1];
     ssize_t n;
@@ -266,7 +264,7 @@ static ssize_t read_fd(int fd, void *ptr, size_t nbytes, int *recvfd) {
     return n;
 }
 
-static ssize_t write_fd(int fd, void *ptr, size_t nbytes, int sendfd) {
+NOEXPORT ssize_t write_fd(int fd, void *ptr, size_t nbytes, int sendfd) {
     struct msghdr msg;
     struct iovec iov[1];
 
