@@ -1,24 +1,24 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
  *   Free Software Foundation; either version 2 of the License, or (at your
  *   option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *   See the GNU General Public License for more details.
- * 
+ *
  *   You should have received a copy of the GNU General Public License along
  *   with this program; if not, see <http://www.gnu.org/licenses>.
- * 
+ *
  *   Linking stunnel statically or dynamically with other modules is making
  *   a combined work based on stunnel. Thus, the terms and conditions of
  *   the GNU General Public License cover the whole combination.
- * 
+ *
  *   In addition, as a special exception, the copyright holder of stunnel
  *   gives you permission to combine stunnel with free software programs or
  *   libraries that are released under the GNU LGPL and with code included
@@ -26,7 +26,7 @@
  *   modified versions of such code, with unchanged license). You may copy
  *   and distribute such a system following the terms of the GNU GPL for
  *   stunnel and the licenses of the other code concerned.
- * 
+ *
  *   Note that people who make modified versions of stunnel are not obligated
  *   to grant this special exception for their modified versions; it is their
  *   choice whether to do so. The GNU General Public License gives permission
@@ -47,31 +47,39 @@
 #endif /* va_copy */
 
 static u8 canary[10]; /* 80-bit canary value */
-static volatile int canary_initialized=0;
+static volatile int canary_initialized=0, str_initialized=0;
 
-typedef struct alloc_list ALLOC_LIST;
+typedef struct alloc_list_struct ALLOC_LIST;
 
-typedef struct alloc_tls {
+typedef struct {
     ALLOC_LIST *head;
     size_t bytes, blocks;
 } ALLOC_TLS;
 
-struct alloc_list {
+struct alloc_list_struct {
     ALLOC_LIST *prev, *next;
     ALLOC_TLS *tls;
     size_t size;
+    char *file;
+    int line;
     int valid_canary;
     unsigned int magic;
+        /* at least on IA64 allocations need to be aligned */
+#ifdef __GNUC__
+} __attribute__((aligned(16)));
+#else
+    int padding[2]; /* the number of integers is architecture-specific */
 };
+#endif
 
-static void set_alloc_tls(ALLOC_TLS *);
-static ALLOC_TLS *get_alloc_tls();
-static ALLOC_LIST *get_alloc_list_ptr(void *, char *, int);
+NOEXPORT void set_alloc_tls(ALLOC_TLS *);
+NOEXPORT ALLOC_TLS *get_alloc_tls();
+NOEXPORT ALLOC_LIST *get_alloc_list_ptr(void *, char *, int);
 
-char *str_dup(const char *str) {
+char *str_dup_debug(const char *str, char *file, int line) {
     char *retval;
 
-    retval=str_alloc(strlen(str)+1);
+    retval=str_alloc_debug(strlen(str)+1, file, line);
     strcpy(retval, str);
     return retval;
 }
@@ -111,16 +119,17 @@ char *str_vprintf(const char *format, va_list start_ap) {
 static ALLOC_TLS *global_tls=NULL;
 
 void str_init() {
+    str_initialized=1;
 }
 
-static void set_alloc_tls(ALLOC_TLS *tls) {
+NOEXPORT void set_alloc_tls(ALLOC_TLS *tls) {
     if(ready_head)
         ready_head->tls=tls;
     else /* ucontext threads not initialized */
         global_tls=tls;
 }
 
-static ALLOC_TLS *get_alloc_tls() {
+NOEXPORT ALLOC_TLS *get_alloc_tls() {
     if(ready_head)
         return ready_head->tls;
     else /* ucontext threads not initialized */
@@ -131,19 +140,18 @@ static ALLOC_TLS *get_alloc_tls() {
 
 #ifdef USE_FORK
 
-static ALLOC_TLS *global_tls;
-static pid_t previous_pid=0;
+static ALLOC_TLS *global_tls=NULL;
 
 void str_init() {
+    str_initialized=1;
 }
 
-static void set_alloc_tls(ALLOC_TLS *tls) {
+NOEXPORT void set_alloc_tls(ALLOC_TLS *tls) {
     global_tls=tls;
-    previous_pid=getpid();
 }
 
-static ALLOC_TLS *get_alloc_tls() {
-    return previous_pid==getpid() ? global_tls : NULL;
+NOEXPORT ALLOC_TLS *get_alloc_tls() {
+    return global_tls;
 }
 
 #endif /* USE_FORK */
@@ -154,13 +162,14 @@ static pthread_key_t pthread_key;
 
 void str_init() {
     pthread_key_create(&pthread_key, NULL);
+    str_initialized=1;
 }
 
-static void set_alloc_tls(ALLOC_TLS *tls) {
+NOEXPORT void set_alloc_tls(ALLOC_TLS *tls) {
     pthread_setspecific(pthread_key, tls);
 }
 
-static ALLOC_TLS *get_alloc_tls() {
+NOEXPORT ALLOC_TLS *get_alloc_tls() {
     return pthread_getspecific(pthread_key);
 }
 
@@ -168,23 +177,26 @@ static ALLOC_TLS *get_alloc_tls() {
 
 #ifdef USE_WIN32
 
-static DWORD tls_index; 
+static DWORD tls_index;
 
 void str_init() {
     tls_index=TlsAlloc();
+    str_initialized=1;
 }
 
-static void set_alloc_tls(ALLOC_TLS *alloc_tls) {
+NOEXPORT void set_alloc_tls(ALLOC_TLS *alloc_tls) {
     TlsSetValue(tls_index, alloc_tls);
 }
 
-static ALLOC_TLS *get_alloc_tls() {
+NOEXPORT ALLOC_TLS *get_alloc_tls() {
     return TlsGetValue(tls_index);
 }
 
 #endif /* USE_WIN32 */
 
 void str_canary_init() {
+    if(canary_initialized) /* prevent double initialization on config reload */
+        return;
     RAND_bytes(canary, sizeof canary);
     canary_initialized=1; /* after RAND_bytes */
 }
@@ -192,10 +204,12 @@ void str_canary_init() {
 void str_cleanup() {
     ALLOC_TLS *alloc_tls;
 
+    if(!str_initialized)
+        fatal_debug("str not initialized", __FILE__, __LINE__);
     alloc_tls=get_alloc_tls();
     if(alloc_tls) {
-        while(alloc_tls->head)
-            str_free(alloc_tls->head+1);
+        while(alloc_tls->head) /* str_free macro requires lvalue parameter */
+            str_free_debug(alloc_tls->head+1, __FILE__, __LINE__);
         set_alloc_tls(NULL);
         free(alloc_tls);
     }
@@ -203,40 +217,58 @@ void str_cleanup() {
 
 void str_stats() {
     ALLOC_TLS *alloc_tls;
+    ALLOC_LIST *alloc_list;
+    int i=0;
 
+    if(!str_initialized)
+        fatal_debug("str not initialized", __FILE__, __LINE__);
     alloc_tls=get_alloc_tls();
-    if(alloc_tls)
-        s_log(LOG_DEBUG,
-            "str_stats: %lu block(s), %lu data byte(s), %lu control byte(s)",
-            (unsigned long int)alloc_tls->blocks,
-            (unsigned long int)alloc_tls->bytes,
-            (unsigned long int)(alloc_tls->blocks*
-                (sizeof(ALLOC_LIST)+sizeof canary)));
-    else
+    if(!alloc_tls) {
         s_log(LOG_DEBUG, "str_stats: alloc_tls not initialized");
+        return;
+    }
+    if(!alloc_tls->blocks && !alloc_tls->bytes)
+        return; /* skip if no data is allocated */
+    s_log(LOG_DEBUG, "str_stats: %lu block(s), "
+            "%lu data byte(s), %lu control byte(s)",
+        (unsigned long int)alloc_tls->blocks,
+        (unsigned long int)alloc_tls->bytes,
+        (unsigned long int)(alloc_tls->blocks*
+            (sizeof(ALLOC_LIST)+sizeof canary)));
+    for(alloc_list=alloc_tls->head; alloc_list; alloc_list=alloc_list->next) {
+        if(++i>10) /* limit the number of results */
+            break;
+        s_log(LOG_DEBUG, "str_stats: %lu byte(s) at %s:%d",
+            (unsigned long int)alloc_list->size,
+            alloc_list->file, alloc_list->line);
+    }
 }
 
 void *str_alloc_debug(size_t size, char *file, int line) {
     ALLOC_TLS *alloc_tls;
     ALLOC_LIST *alloc_list;
 
+    if(!str_initialized)
+        fatal_debug("str not initialized", file, line);
     alloc_tls=get_alloc_tls();
     if(!alloc_tls) { /* first allocation in this thread */
         alloc_tls=calloc(1, sizeof(ALLOC_TLS));
         if(!alloc_tls)
-            fatal("Out of memory", file, line);
+            fatal_debug("Out of memory", file, line);
         alloc_tls->head=NULL;
         alloc_tls->bytes=alloc_tls->blocks=0;
         set_alloc_tls(alloc_tls);
     }
     alloc_list=calloc(1, sizeof(ALLOC_LIST)+size+sizeof canary);
     if(!alloc_list)
-        fatal("Out of memory", file, line);
+        fatal_debug("Out of memory", file, line);
 
     alloc_list->prev=NULL;
     alloc_list->next=alloc_tls->head;
     alloc_list->tls=alloc_tls;
     alloc_list->size=size;
+    alloc_list->file=file;
+    alloc_list->line=line;
     alloc_list->valid_canary=canary_initialized; /* before memcpy */
     memcpy((u8 *)(alloc_list+1)+size, canary, sizeof canary);
     alloc_list->magic=0xdeadbeef;
@@ -251,18 +283,26 @@ void *str_alloc_debug(size_t size, char *file, int line) {
 }
 
 void *str_realloc_debug(void *ptr, size_t size, char *file, int line) {
-    ALLOC_LIST *previous_alloc_list, *alloc_list;
+    ALLOC_LIST *prev_alloc_list, *alloc_list;
 
     if(!ptr)
-        return str_alloc(size);
-    previous_alloc_list=get_alloc_list_ptr(ptr, file, line);
-    alloc_list=realloc(previous_alloc_list,
+        return str_alloc_debug(size, file, line);
+    prev_alloc_list=get_alloc_list_ptr(ptr, file, line);
+    if(prev_alloc_list->size>size) /* shrinking the allocation */
+        memset((u8 *)ptr+size, 0, prev_alloc_list->size-size); /* paranoia */
+    alloc_list=realloc(prev_alloc_list,
         sizeof(ALLOC_LIST)+size+sizeof canary);
     if(!alloc_list)
-        fatal("Out of memory", file, line);
+        fatal_debug("Out of memory", file, line);
+    ptr=alloc_list+1;
+    if(size>alloc_list->size) /* growing the allocation */
+        memset((u8 *)ptr+alloc_list->size, 0, size-alloc_list->size);
+    alloc_list->size=size;
+    alloc_list->file=file;
+    alloc_list->line=line;
     if(alloc_list->tls) { /* not detached */
         /* refresh possibly invalidated linked list pointers */
-        if(alloc_list->tls->head==previous_alloc_list)
+        if(alloc_list->tls->head==prev_alloc_list)
             alloc_list->tls->head=alloc_list;
         if(alloc_list->next)
             alloc_list->next->prev=alloc_list;
@@ -271,10 +311,9 @@ void *str_realloc_debug(void *ptr, size_t size, char *file, int line) {
         /* update statistics */
         alloc_list->tls->bytes+=size-alloc_list->size;
     }
-    alloc_list->size=size;
     alloc_list->valid_canary=canary_initialized; /* before memcpy */
-    memcpy((u8 *)(alloc_list+1)+size, canary, sizeof canary);
-    return alloc_list+1;
+    memcpy((u8 *)ptr+size, canary, sizeof canary);
+    return ptr;
 }
 
 /* detach from thread automatic deallocation list */
@@ -310,22 +349,38 @@ void str_free_debug(void *ptr, char *file, int line) {
         return;
     str_detach_debug(ptr, file, line);
     alloc_list=(ALLOC_LIST *)ptr-1;
-    alloc_list->magic=0xdefec8ed; /* to detect double free */
+    alloc_list->magic=0xdefec8ed; /* to detect double free attempts */
+    memset(ptr, 0, alloc_list->size); /* paranoia */
     free(alloc_list);
 }
 
-static ALLOC_LIST *get_alloc_list_ptr(void *ptr, char *file, int line) {
+NOEXPORT ALLOC_LIST *get_alloc_list_ptr(void *ptr, char *file, int line) {
     ALLOC_LIST *alloc_list;
 
+    if(!str_initialized)
+        fatal_debug("str not initialized", file, line);
     alloc_list=(ALLOC_LIST *)ptr-1;
-    if(alloc_list->magic!=0xdeadbeef) /* not allocated by str_alloc() */
-        fatal("Bad magic", file, line);
+    if(alloc_list->magic!=0xdeadbeef) { /* not allocated by str_alloc() */
+        if(alloc_list->magic==0xdefec8ed)
+            fatal_debug("Double free attempt", file, line);
+        else
+            fatal_debug("Bad magic", file, line); /* LOL */
+    }
     if(alloc_list->tls /* not detached */ && alloc_list->tls!=get_alloc_tls())
-        fatal("Wrong thread", file, line);
+        fatal_debug("Memory allocated in a different thread", file, line);
     if(alloc_list->valid_canary &&
-            memcmp((u8 *)ptr+alloc_list->size, canary, sizeof canary))
-        fatal("Dead canary", file, line);
+            safe_memcmp((u8 *)ptr+alloc_list->size, canary, sizeof canary))
+        fatal_debug("Dead canary", file, line); /* LOL */
     return alloc_list;
 }
 
+/* a version of memcmp() with execution time not dependent on data values */
+/* it does *not* allow to test wheter s1 is greater or lesser than s2  */
+int safe_memcmp(const void *s1, const void *s2, size_t n) {
+    u8 *p1=(u8 *)s1, *p2=(u8 *)s2, r=0;
+    while(n--)
+        r|=*p1++^*p2++;
+    return r;
+}
+
 /* end of str.c */
